/*
 * This file is part of the KFTPGrabber project
 *
 * Copyright (C) 2003-2004 by the KFTPGrabber developers
 * Copyright (C) 2003-2004 Jernej Kos <kostko@jweb-network.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
 * NON-INFRINGEMENT.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */

#ifndef KFTPQUEUE_H
#define KFTPQUEUE_H

#include <qstring.h>
#include <qvaluelist.h>
#include <qtimer.h>
#include <qcache.h>

#include <kurl.h>
#include <kurldrag.h>

#include "kftpclientthread.h"
#include "kftpqueueprocessor.h"
#include "kftpqueueconverter.h"
#include "misc.h"

#include "kftptransfer.h"
#include "kftptransferfile.h"
#include "kftptransferdir.h"
#include "site.h"

namespace KFTPNetwork {
  class Socket;
}

using namespace KFTPGrabberBase;

class KFTPSession;

typedef QPtrList<KFTPQueue::Transfer> KFTPQueueTransfers;

namespace KFTPQueue {

#define QS_THR_EVENT_ID 65200

class QueueScannerThreadEvent : public QCustomEvent
{
public:
    QueueScannerThreadEvent(void *data)
      : QCustomEvent((QEvent::Type) QS_THR_EVENT_ID, data) {}
};

class QueueScannerThread : public QThread {
public:
    QueueScannerThread(KFTPSession *session, Transfer *item);
    QueueScannerThread() : QThread() {};
protected:
    void run();
private:
    KFTPSession *m_session;
    Transfer *m_item;
    
    void scanFolder(Transfer *parent);
};

class Manager : public QObject {
Q_OBJECT
friend class KFTPSession;
friend class ::KFTPQueueConverter;
friend class KFTPNetwork::Socket;
friend class KFTPQueue::QueueScannerThread;
friend class KFTPQueue::FailedTransfer;
public:
    static Manager *self();
    ~Manager();
    
    QueueObject *topLevelObject() { return m_topLevel; }
    
    void insertTransfer(KURLDrag *drag);
    void insertTransfer(Transfer *transfer);
    
    void removeTransfer(Transfer *transfer);
    void removeTransfer(long id);
    void clearQueue();
    
    /**
     * Check if the transfer is under the correct site and move it if not.
     *
     * @param transfer The transfer to check
     */
    void revalidateTransfer(Transfer *transfer);
    
    Transfer *findTransfer(long id);
    Site *findSite(KURL url);
    
    void removeFailedTransfer(FailedTransfer *transfer);
    void clearFailedTransferList();
    
    void moveTransferUp(long id);
    void moveTransferDown(long id);
    
    bool canBeMovedUp(long id);
    bool canBeMovedDown(long id);
    
    QPtrList<KFTPQueue::FailedTransfer> *getFailedTransferList() { return &m_failedTransfers; }
    KFTPQueueConverter *getConverter() { return m_converter; }
    
    void setEmitUpdate(bool value) { m_emitUpdate = value; }
    void doEmitUpdate();

    filesize_t getDownloadSpeed() { return m_curDownSpeed; }
    filesize_t getUploadSpeed() { return m_curUpSpeed; }
    
    int getTransferPercentage();
    int getNumRunning(bool onlyDirs = false);
    
    void start();
    void abort();
    bool isProcessing() { return m_queueProc->isRunning(); }
    
    static int fileExistsAction(KFTPQueue::TransferFile *transfer,
                                KFTPQueue::TransferType type,
                                const KURL &src,
                                const KURL &dst,
                                const FTPEntry &srcStat = FTPEntry(),
                                const FTPEntry &dstStat = FTPEntry() );
                                
    KFTPQueue::Transfer *spawnTransfer(KURL srcURL, KURL dstURL, filesize_t size, bool dir, KFTPQueue::TransferType t_type, 
                                       bool ignoreSkip = false, bool insertToQueue = true, QObject *parent = 0L, bool noScan = false);
protected:
    Manager();
    static Manager *m_self;
private:
    QueueObject *m_topLevel;
    QCache<QueueObject> m_queueObjectCache;
    
    QPtrList<KFTPQueue::FailedTransfer> m_failedTransfers;
    KFTPQueueProcessor *m_queueProc;
    KFTPQueueConverter *m_converter;
    
    int m_lastQID;
    bool m_emitUpdate;
    bool m_processingQueue;

    filesize_t m_curDownSpeed;
    filesize_t m_curUpSpeed;
private slots:
    void slotQueueProcessingComplete();
    void slotQueueProcessingAborted();
signals:
    void newSite(KFTPQueue::Site*);
    void newTransfer(KFTPQueue::Transfer*);
    
    void transferRemoved(long);
    void siteRemoved(long);
    
    void queueUpdate();
    
    void failedTransferNew(KFTPQueue::FailedTransfer*);
    void failedTransferRemoved(long);
};

}

#endif
